@@ -1,5 +1,6 @@
 from operator import add
-from itertools import islice, ifilter, imap, izip, chain, tee, ifilterfalse, dropwhile, takewhile
+from itertools import islice, ifilter, imap, izip, chain, tee, ifilterfalse, dropwhile, takewhile, \
+                      groupby
 from collections import defaultdict, Sequence
 
 from .funcs import partial
@@ -13,7 +14,9 @@ __all__ = [
     'concat', 'iconcat', 'chain', 'cat', 'icat', 'mapcat', 'imapcat',
     'izip', 'interleave', 'interpose', 'distinct',
     'dropwhile', 'takewhile', 'split', 'split_at', 'split_by',
-    'group_by', 'partition', 'ipartition', 'chunks', 'ichunks', 'with_prev',
+    'group_by', 'count_by',
+    'partition', 'ipartition', 'chunks', 'ichunks', 'ipartition_by', 'partition_by',
+    'with_prev',
     'ireductions', 'reductions', 'isums', 'sums',
 ]
 
@@ -145,6 +148,12 @@ def group_by(f, seq):
         result[f(item)].append(item)
     return result
 
+def count_by(f, seq):
+    result = defaultdict(int)
+    for item in seq:
+        result[f(item)] += 1
+    return result
+
 
 # For efficiency we use separate implementation for cutting sequences (those capable of slicing)
 def _icut_seq(drop_tail, n, step, seq):
@@ -184,6 +193,13 @@ def ichunks(n, step, seq=EMPTY):
 def chunks(n, step, seq=EMPTY):
     return list(ichunks(n, step, seq))
 
+def ipartition_by(f, seq):
+    for g, items in groupby(seq, f):
+        yield items
+
+def partition_by(f, seq):
+    return map(list, ipartition_by(f, seq))
+
 
 def with_prev(seq):
     a, b = tee(seq)
